MEMORY {
	VECTOR_RESET : ORIGIN = 0xa0200200, LENGTH = 0x00000200
	VECTOR_TLB   : ORIGIN = 0x80000000, LENGTH = 0x00000080
	VECTOR_XTLB  : ORIGIN = 0x80000080, LENGTH = 0x00000080
	VECTOR_CACHE : ORIGIN = 0xa0000100, LENGTH = 0x00000080
	VECTOR_OTHER : ORIGIN = 0x80000180, LENGTH = 0x00000080
	RO_KSEG1     : ORIGIN = 0xa0200400, LENGTH = 0x0000fc00
	RO_KSEG0     : ORIGIN = 0x80280000, LENGTH = 0x00010000
	RO_KERNEL    : ORIGIN = 0x00290000, LENGTH = 0x00020000
	RO_DOM1      : ORIGIN = 0x002b0000, LENGTH = 0x00010000
	RO_DOM2      : ORIGIN = 0x002c0000, LENGTH = 0x00010000
	RO_DOM3      : ORIGIN = 0x002d0000, LENGTH = 0x00010000
	RO_SHARED    : ORIGIN = 0x002e0000, LENGTH = 0x00020000
	RW_KSEG1     : ORIGIN = 0xa0300000, LENGTH = 0x00010000
	RW_KSEG0     : ORIGIN = 0x80380000, LENGTH = 0x00010000
	RW_KERNEL    : ORIGIN = 0x00390000, LENGTH = 0x00020000
	RW_DOM1      : ORIGIN = 0x003b0000, LENGTH = 0x00010000
	RW_DOM2      : ORIGIN = 0x003c0000, LENGTH = 0x00010000
	RW_DOM3      : ORIGIN = 0x003d0000, LENGTH = 0x00010000
	RW_SHARED    : ORIGIN = 0x003e0000, LENGTH = 0x00020000
}

NOCROSSREFS(.cpu_memory .text_shared)
NOCROSSREFS(.cpu_memory .rodata_shared)
NOCROSSREFS(.cpu_memory .data_shared)
NOCROSSREFS(.cpu_memory .bss_shared)

PROVIDE(_gp = 0);
PROVIDE(hardware_init_hook = 0);
PROVIDE(software_init_hook = 0);
PROVIDE(software_term_hook = 0);

SECTIONS {
	.vector_reset : {
		*(.vector_reset)
	} > VECTOR_RESET
	.vector_tlb : {
		*(.vector_tlb)
	} > VECTOR_TLB
	.vector_xtlb : {
		*(.vector_xtlb)
	} > VECTOR_XTLB
	.vector_cache : {
		*(.vector_cache)
	} > VECTOR_CACHE
	.vector_other : {
		*(.vector_other)
	} > VECTOR_OTHER

	.text_kseg1 : {
		*(.text_reset)
	} > RO_KSEG1

	.text_kseg0 : {
		*(.text_unmapped)
	} > RO_KSEG0
	.cpu_memory : {
		cpu_memory.o(.text)
		cpu_memory.o(.rodata)
	} > RO_KSEG0
	.rodata_kseg0 : {
		alarm.o(.rodata)
		banner.o(.rodata)
		cyclic.o(.rodata)
		dataqueue.o(.rodata)
		domain.o(.rodata)
		eventflag.o(.rodata)
		exception.o(.rodata)
		interrupt.o(.rodata)
		mailbox.o(.rodata)
		memory.o(.rodata)
		mempfix.o(.rodata)
		mutex.o(.rodata)
		overrun.o(.rodata)
		semaphore.o(.rodata)
		startup.o(.rodata)
		svc_manage.o(.rodata)
		sys_manage.o(.rodata)
		syslog.o(.rodata)
		task.o(.rodata)
		task_except.o(.rodata)
		task_manage.o(.rodata)
		task_refer.o(.rodata)
		task_sync.o(.rodata)
		time_event.o(.rodata)
		time_manage.o(.rodata)
		wait.o(.rodata)
		kernel_cfg.o(.rodata)
		cpu_config.o(.rodata)
		hw_ovrtimer.o(.rodata)
		hw_timer.o(.rodata)
		sys_config.o(.rodata)
	} > RO_KSEG0
	_sidata_unmapped = .;
	.data_kseg0 : AT(_sidata_unmapped) {
		_sdata_unmapped = .;
		alarm.o(.data)
		banner.o(.data)
		cyclic.o(.data)
		dataqueue.o(.data)
		domain.o(.data)
		eventflag.o(.data)
		exception.o(.data)
		interrupt.o(.data)
		mailbox.o(.data)
		memory.o(.data)
		mempfix.o(.data)
		mutex.o(.data)
		overrun.o(.data)
		semaphore.o(.data)
		startup.o(.data)
		svc_manage.o(.data)
		sys_manage.o(.data)
		syslog.o(.data)
		task.o(.data)
		task_except.o(.data)
		task_manage.o(.data)
		task_refer.o(.data)
		task_sync.o(.data)
		time_event.o(.data)
		time_manage.o(.data)
		wait.o(.data)
		kernel_cfg.o(.data)
		cpu_config.o(.data)
		hw_ovrtimer.o(.data)
		hw_timer.o(.data)
		sys_config.o(.data)
	} > RW_KSEG0
	_eidata_unmapped = _sidata_unmapped + SIZEOF(.data_kseg0);
	.bss_kseg0 : {
		_sbss_unmapped = .;
		alarm.o(.bss COMMON)
		banner.o(.bss COMMON)
		cyclic.o(.bss COMMON)
		dataqueue.o(.bss COMMON)
		domain.o(.bss COMMON)
		eventflag.o(.bss COMMON)
		exception.o(.bss COMMON)
		interrupt.o(.bss COMMON)
		mailbox.o(.bss COMMON)
		memory.o(.bss COMMON)
		mempfix.o(.bss COMMON)
		mutex.o(.bss COMMON)
		overrun.o(.bss COMMON)
		semaphore.o(.bss COMMON)
		startup.o(.bss COMMON)
		svc_manage.o(.bss COMMON)
		sys_manage.o(.bss COMMON)
		syslog.o(.bss COMMON)
		task.o(.bss COMMON)
		task_except.o(.bss COMMON)
		task_manage.o(.bss COMMON)
		task_refer.o(.bss COMMON)
		task_sync.o(.bss COMMON)
		time_event.o(.bss COMMON)
		time_manage.o(.bss COMMON)
		wait.o(.bss COMMON)
		kernel_cfg.o(.bss COMMON)
		cpu_config.o(.bss COMMON)
		hw_ovrtimer.o(.bss COMMON)
		hw_timer.o(.bss COMMON)
		sys_config.o(.bss COMMON)
		_ebss_unmapped = .;
	} > RW_KSEG0

	.text_kernel : {
		alarm.o(.text)
		banner.o(.text)
		cyclic.o(.text)
		dataqueue.o(.text)
		domain.o(.text)
		eventflag.o(.text)
		exception.o(.text)
		interrupt.o(.text)
		mailbox.o(.text)
		memory.o(.text)
		mempfix.o(.text)
		mutex.o(.text)
		overrun.o(.text)
		semaphore.o(.text)
		startup.o(.text)
		svc_manage.o(.text)
		sys_manage.o(.text)
		syslog.o(.text)
		task.o(.text)
		task_except.o(.text)
		task_manage.o(.text)
		task_refer.o(.text)
		task_sync.o(.text)
		time_event.o(.text)
		time_manage.o(.text)
		wait.o(.text)
		kernel_cfg.o(.text)
		cpu_config.o(.text)
		hw_ovrtimer.o(.text)
		hw_timer.o(.text)
		sys_config.o(.text)
		sample.o(.text)
	} > RO_KERNEL
	.rodata_kernel : {
		sample.o(.rodata)
	} > RO_KERNEL
	_sidata_kernel = .;
	.data_kernel : AT(_sidata_kernel) {
		_sdata_kernel = .;
		sample.o(.data)
	} > RW_KERNEL
	_eidata_kernel = _sidata_kernel + SIZEOF(.data_kernel);
	.bss_kernel : {
		_sbss_kernel = .;
		sample.o(.bss COMMON)
		_ebss_kernel = .;
	} > RW_KERNEL

	.text_DOM1 : {
		sample1.o(.text)
	} > RO_DOM1
	.rodata_DOM1 : {
		sample1.o(.rodata)
	} > RO_DOM1
	_sidata_DOM1 = .;
	.data_DOM1 : AT(_sidata_DOM1) {
		_sdata_DOM1 = .;
		sample1.o(.data)
	} > RW_DOM1
	_eidata_DOM1 = _sidata_DOM1 + SIZEOF(.data_DOM1);
	.bss_DOM1 : {
		_sbss_DOM1 = .;
		sample1.o(.bss COMMON)
		_ebss_DOM1 = .;
	} > RW_DOM1

	.text_DOM2 : {
		sample2.o(.text)
	} > RO_DOM2
	.rodata_DOM2 : {
		sample2.o(.rodata)
	} > RO_DOM2
	_sidata_DOM2 = .;
	.data_DOM2 : AT(_sidata_DOM2) {
		_sdata_DOM2 = .;
		sample2.o(.data)
	} > RW_DOM2
	_eidata_DOM2 = _sidata_DOM2 + SIZEOF(.data_DOM2);
	.bss_DOM2 : {
		_sbss_DOM2 = .;
		sample2.o(.bss COMMON)
		_ebss_DOM2 = .;
	} > RW_DOM2

	.text_DOM3 : {
		sample3.o(.text)
	} > RO_DOM3
	.rodata_DOM3 : {
		sample3.o(.rodata)
	} > RO_DOM3
	_sidata_DOM3 = .;
	.data_DOM3 : AT(_sidata_DOM3) {
		_sdata_DOM3 = .;
		sample3.o(.data)
	} > RW_DOM3
	_eidata_DOM3 = _sidata_DOM3 + SIZEOF(.data_DOM3);
	.bss_DOM3 : {
		_sbss_DOM3 = .;
		sample3.o(.bss COMMON)
		_ebss_DOM3 = .;
	} > RW_DOM3

	.text_shared : {
		*(.text)
	} > RO_SHARED
	.rodata_shared : {
		*(.rodata)
	} > RO_SHARED
	_sidata_shared = .;
	.data_shared : AT(_sidata_shared) {
		_sdata_shared = .;
		*(.data)
	} > RW_SHARED
	_eidata_shared = _sidata_shared + SIZEOF(.data_shared);
	.bss_shared : {
		_sbss_shared = .;
		*(.bss COMMON)
		_ebss_shared = .;
	} > RW_SHARED

	/DISCARD/ : {
		*(.rel.dyn)
	}

  .reginfo : { *(.reginfo) } > RO_KSEG0
  .eh_frame : { *(.eh_frame) } > RO_KSEG0

  /* DWARF debug sections.
     Symbols in the DWARF debugging sections are relative to 
     the beginning of the section so we begin them at 0.  */

  /* DWARF 1 */
  .debug          0 : { *(.debug) }
  .line           0 : { *(.line) }

  /* GNU DWARF 1 extensions */
  .debug_srcinfo  0 : { *(.debug_srcinfo) }
  .debug_sfnames  0 : { *(.debug_sfnames) }

  /* DWARF 1.1 and DWARF 2 */
  .debug_aranges  0 : { *(.debug_aranges) }
  .debug_pubnames 0 : { *(.debug_pubnames) }

  /* DWARF 2 */
  .debug_info     0 : { *(.debug_info) }
  .debug_abbrev   0 : { *(.debug_abbrev) }
  .debug_line     0 : { *(.debug_line) }
  .debug_frame    0 : { *(.debug_frame) }
  .debug_str      0 : { *(.debug_str) }
  .debug_loc      0 : { *(.debug_loc) }
  .debug_macinfo  0 : { *(.debug_macinfo) }
  .debug_ranges   0 : { *(.debug_ranges) }

  /* SGI/MIPS DWARF 2 extensions */
  .debug_weaknames 0 : { *(.debug_weaknames) }
  .debug_funcnames 0 : { *(.debug_funcnames) }
  .debug_typenames 0 : { *(.debug_typenames) }
  .debug_varnames  0 : { *(.debug_varnames) }
}
